@@ -32,6 +32,7 @@ class TestXcbWindow : public QObject
 {
     Q_OBJECT
 private Q_SLOTS:
+    void initTestCase();
     void defaultCtor();
     void ctor();
     void classCtor();
@@ -42,6 +43,12 @@ private Q_SLOTS:
     void destroyNotManaged();
 };
 
+void TestXcbWindow::initTestCase()
+{
+    qApp->setProperty("x11RootWindow", QVariant::fromValue<quint32>(QX11Info::appRootWindow()));
+    qApp->setProperty("x11Connection", QVariant::fromValue<void*>(QX11Info::connection()));
+}
+
 void TestXcbWindow::defaultCtor()
 {
     Xcb::Window window;
