@@ -40,6 +40,10 @@ static const QByteArray s_blurAtomName = QByteArrayLiteral("_KDE_NET_WM_BLUR_BEH
 BlurEffect::BlurEffect()
 {
     shader = BlurShader::create();
+    m_simpleShader = ShaderManager::instance()->generateShaderFromResources(ShaderTrait::MapTexture, QString(), QStringLiteral("logout-blur.frag"));
+    if (!m_simpleShader->isValid()) {
+        qCDebug(KWINEFFECTS) << "Simple blur shader failed to load";
+    }
 
     // Offscreen texture that's used as the target for the horizontal blur pass
     // and the source for the vertical pass.
@@ -78,6 +82,7 @@ BlurEffect::~BlurEffect()
 {
     windows.clear();
 
+    delete m_simpleShader;
     delete shader;
     delete target;
 }
@@ -412,7 +417,7 @@ bool BlurEffect::shouldBlur(const EffectWindow *w, int mask, const WindowPaintDa
     bool scaled = !qFuzzyCompare(data.xScale(), 1.0) && !qFuzzyCompare(data.yScale(), 1.0);
     bool translated = data.xTranslation() || data.yTranslation();
 
-    if (scaled || ((translated || (mask & PAINT_WINDOW_TRANSFORMED)) && !w->data(WindowForceBlurRole).toBool()))
+    if ((scaled || (translated || (mask & PAINT_WINDOW_TRANSFORMED))) && !w->data(WindowForceBlurRole).toBool())
         return false;
 
     bool blurBehindDecos = effects->decorationsHaveAlpha() &&
@@ -430,18 +435,36 @@ void BlurEffect::drawWindow(EffectWindow *w, int mask, QRegion region, WindowPai
     if (shouldBlur(w, mask, data)) {
         QRegion shape = region & blurRegion(w).translated(w->pos()) & screen;
 
-        const bool translated = data.xTranslation() || data.yTranslation();
         // let's do the evil parts - someone wants to blur behind a transformed window
-        if (translated) {
+        const bool translated = data.xTranslation() || data.yTranslation();
+        const bool scaled = data.xScale() != 1 || data.yScale() != 1;
+        if (scaled) {
+            QPoint pt = shape.boundingRect().topLeft();
+            QVector<QRect> shapeRects = shape.rects();
+            shape = QRegion(); // clear
+            foreach (QRect r, shapeRects) {
+                r.moveTo(pt.x() + (r.x() - pt.x()) * data.xScale() + data.xTranslation(),
+                            pt.y() + (r.y() - pt.y()) * data.yScale() + data.yTranslation());
+                r.setWidth(r.width() * data.xScale());
+                r.setHeight(r.height() * data.yScale());
+                shape |= r;
+            }
+            shape = shape & region;
+
+        //Only translated, not scaled
+        } else if (translated) {
             shape = shape.translated(data.xTranslation(), data.yTranslation());
             shape = shape & region;
         }
 
         if (!shape.isEmpty()) {
-            if (m_shouldCache && !translated && !w->isDeleted()) {
-                doCachedBlur(w, region, data.opacity());
+            if (w->isFullScreen() && GLRenderTarget::blitSupported() && m_simpleShader->isValid()
+                    && !GLPlatform::instance()->supports(LimitedNPOT) && shape.boundingRect() == w->geometry()) {
+                doSimpleBlur(w, data.opacity(), data.screenProjectionMatrix());
+            } else if (m_shouldCache && !translated && !w->isDeleted()) {
+                doCachedBlur(w, region, data.opacity(), data.screenProjectionMatrix());
             } else {
-                doBlur(shape, screen, data.opacity());
+                doBlur(shape, screen, data.opacity(), data.screenProjectionMatrix());
             }
         }
     }
@@ -456,12 +479,37 @@ void BlurEffect::paintEffectFrame(EffectFrame *frame, QRegion region, double opa
     bool valid = target->valid() && shader && shader->isValid();
     QRegion shape = frame->geometry().adjusted(-5, -5, 5, 5) & screen;
     if (valid && !shape.isEmpty() && region.intersects(shape.boundingRect()) && frame->style() != EffectFrameNone) {
-        doBlur(shape, screen, opacity * frameOpacity);
+        doBlur(shape, screen, opacity * frameOpacity, frame->screenProjectionMatrix());
     }
     effects->paintEffectFrame(frame, region, opacity, frameOpacity);
 }
 
-void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float opacity)
+void BlurEffect::doSimpleBlur(EffectWindow *w, const float opacity, const QMatrix4x4 &screenProjection)
+{
+    // The fragment shader uses a LOD bias of 1.75, so we need 3 mipmap levels.
+    GLTexture blurTexture = GLTexture(GL_RGBA8, w->size(), 3);
+    blurTexture.setFilter(GL_LINEAR_MIPMAP_LINEAR);
+    blurTexture.setWrapMode(GL_CLAMP_TO_EDGE);
+
+    target->attachTexture(blurTexture);
+    target->blitFromFramebuffer(w->geometry(), QRect(QPoint(0, 0), w->size()));
+
+    // Unmodified base image
+    ShaderBinder binder(m_simpleShader);
+    QMatrix4x4 mvp = screenProjection;
+    mvp.translate(w->x(), w->y());
+    m_simpleShader->setUniform(GLShader::ModelViewProjectionMatrix, mvp);
+    m_simpleShader->setUniform("u_alphaProgress", opacity);
+    glEnable(GL_BLEND);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    blurTexture.bind();
+    blurTexture.generateMipmaps();
+    blurTexture.render(infiniteRegion(), w->geometry());
+    blurTexture.unbind();
+    glDisable(GL_BLEND);
+}
+
+void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float opacity, const QMatrix4x4 &screenProjection)
 {
     const QRegion expanded = expand(shape) & screen;
     const QRect r = expanded.boundingRect();
@@ -489,6 +537,10 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
     shader->setDirection(Qt::Horizontal);
     shader->setPixelDistance(1.0 / r.width());
 
+    QMatrix4x4 modelViewProjectionMatrix;
+    modelViewProjectionMatrix.ortho(0, tex.width(), tex.height(), 0 , 0, 65535);
+    shader->setModelViewProjectionMatrix(modelViewProjectionMatrix);
+
     // Set up the texture matrix to transform from screen coordinates
     // to texture coordinates.
     QMatrix4x4 textureMatrix;
@@ -529,6 +581,7 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
     textureMatrix.scale(1.0 / tex.width(), -1.0 / tex.height(), 1);
     textureMatrix.translate(0, -tex.height(), 0);
     shader->setTextureMatrix(textureMatrix);
+    shader->setModelViewProjectionMatrix(screenProjection);
 
     vbo->draw(GL_TRIANGLES, expanded.rectCount() * 6, shape.rectCount() * 6);
     vbo->unbindArrays();
@@ -541,7 +594,7 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
     shader->unbind();
 }
 
-void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const float opacity)
+void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const float opacity, const QMatrix4x4 &screenProjection)
 {
     const QRect screen = effects->virtualScreenGeometry();
     const QRegion blurredRegion = blurRegion(w).translated(w->pos()) & screen;
@@ -671,10 +724,7 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
         glBlendFunc(GL_CONSTANT_ALPHA, GL_ONE_MINUS_CONSTANT_ALPHA);
     }
 
-    modelViewProjectionMatrix.setToIdentity();
-    const QSize screenSize = effects->virtualScreenSize();
-    modelViewProjectionMatrix.ortho(0, screenSize.width(), screenSize.height(), 0, 0, 65535);
-    shader->setModelViewProjectionMatrix(modelViewProjectionMatrix);
+    shader->setModelViewProjectionMatrix(screenProjection);
 
     // Set the up the texture matrix to transform from screen coordinates
     // to texture coordinates.
