@@ -81,6 +81,7 @@ inline bool KWINGLUTILS_EXPORT isPowerOfTwo(int x)
  * @return power of two integer _greater or equal to_ x.
  *  E.g. nearestPowerOfTwo(513) = nearestPowerOfTwo(800) = 1024
  **/
+// TODO: Drop for Plasma 6, no longer needed after OpenGL 2.0
 int KWINGLUTILS_EXPORT nearestPowerOfTwo(int x);
 
 class KWINGLUTILS_EXPORT GLShader
@@ -227,40 +228,6 @@ Q_DECLARE_FLAGS(ShaderTraits, ShaderTrait)
 class KWINGLUTILS_EXPORT ShaderManager
 {
 public:
-    /**
-     * Identifiers for built-in shaders available for effects and scene
-     **/
-    enum ShaderType {
-        /**
-         * An orthographic projection shader able to render textured geometries.
-         * Expects a @c vec2 uniform @c offset describing the offset from top-left corner
-         * and defaults to @c (0/0). Expects a @c vec2 uniform @c textureSize to calculate
-         * normalized texture coordinates. Defaults to @c (1.0/1.0). And expects a @c vec3
-         * uniform @c colorManiuplation, with @c x being opacity, @c y being brightness and
-         * @c z being saturation. All three values default to @c 1.0.
-         * The sampler uniform is @c sample and defaults to @c 0.
-         * The shader uses two vertex attributes @c vertex and @c texCoord.
-         **/
-        SimpleShader = 0,
-        /**
-         * A generic shader able to render transformed, textured geometries.
-         * This shader is mostly needed by the scene and not of much interest for effects.
-         * Effects can influence this shader through @link ScreenPaintData and @link WindowPaintData.
-         * The shader expects four @c mat4 uniforms @c projection, @c modelview,
-         * @c screenTransformation and @c windowTransformation. The fragment shader expect the
-         * same uniforms as the SimpleShader and the same vertex attributes are used.
-         **/
-        GenericShader,
-        /**
-         * An orthographic shader to render simple colored geometries without texturing.
-         * Expects a @c vec2 uniform @c offset describing the offset from top-left corner
-         * and defaults to @c (0/0). The fragment shader expects a single @c vec4 uniform
-         * @c geometryColor, which defaults to fully opaque black.
-         * The Shader uses one vertex attribute @c vertex.
-         **/
-        ColorShader
-    };
-
     /**
      * Returns a shader with the given traits, creating it if necessary.
      */
@@ -275,13 +242,6 @@ public:
      * @return @c true if a shader is bound, @c false otherwise
      **/
     bool isShaderBound() const;
-    /**
-     * Allows to query whether Shaders are supported by the compositor, that is
-     * whether the Shaders compiled successfully.
-     *
-     * @return @c true if the built-in shaders are valid, @c false otherwise
-     **/
-    bool isValid() const;
     /**
      * Is @c true if the environment variable KWIN_GL_DEBUG is set to 1.
      * In that case shaders are compiled with KWIN_SHADER_DEBUG defined.
@@ -290,16 +250,6 @@ public:
      **/
     bool isShaderDebug() const;
 
-    /**
-     * Binds the shader of specified @p type.
-     * To unbind the shader use @link popShader. A previous bound shader will be rebound.
-     * @param type The built-in shader to bind
-     * @param reset Whether all uniforms should be reset to their default values
-     * @return The bound shader or @c NULL if shaders are not valid
-     * @see popShader
-     **/
-    GLShader *pushShader(ShaderType type, bool reset = false);
-
     /**
      * Pushes the current shader onto the stack and binds a shader
      * with the given traits.
@@ -324,33 +274,6 @@ public:
      **/
     void popShader();
 
-    /**
-     * Resets all shaders to the default uniform values.
-     * Only built in shaders are changed.
-     * @since 4.8
-     **/
-    void resetAllShaders();
-
-    /**
-     * Resets ShaderType @p type uniforms of a custom shader
-     * @since 4.11.1
-     */
-    void resetShader(GLShader *shader, ShaderType type);
-
-    /**
-     * Creates a GLShader with a built-in vertex shader and a custom fragment shader.
-     * @param vertex The generic vertex shader
-     * @param fragmentFile The path to the source code of the fragment shader
-     * @return The created shader
-     **/
-    GLShader *loadFragmentShader(ShaderType vertex, const QString &fragmentFile);
-    /**
-     * Creates a GLShader with a built-in fragment shader and a custom vertex shader.
-     * @param fragment The generic fragment shader
-     * @param vertexFile The path to the source code of the vertex shader
-     * @return The created shader
-     **/
-    GLShader *loadVertexShader(ShaderType fragment, const QString &vertexFile);
     /**
      * Creates a GLShader with the specified sources.
      * The difference to GLShader is that it does not need to be loaded from files.
@@ -360,20 +283,53 @@ public:
      **/
     GLShader *loadShaderFromCode(const QByteArray &vertexSource, const QByteArray &fragmentSource);
 
+    /**
+     * Creates a custom shader with the given @p traits and custom @p vertexSource and or @p fragmentSource.
+     * If the @p vertexSource is empty a vertex shader with the given @p traits is generated.
+     * If it is not empty the @p vertexSource is used as the source for the vertex shader.
+     *
+     * The same applies for argument @p fragmentSource just for the fragment shader.
+     *
+     * So if both @p vertesSource and @p fragmentSource are provided the @p traits are ignored.
+     * If neither are provided a new shader following the @p traits is generated.
+     *
+     * @param traits The shader traits for generating the shader
+     * @param vertesSource optional vertex shader source code to be used instead of shader traits
+     * @param fragmentSource optional fragment shader source code to be used instead of shader traits
+     * @return new generated shader
+     * @since 5.6
+     **/
+    GLShader *generateCustomShader(ShaderTraits traits, const QByteArray &vertexSource = QByteArray(), const QByteArray &fragmentSource = QByteArray());
+
+    /**
+     * Creates a custom shader with the given @p traits and custom @p vertexFile and or @p fragmentFile.
+     * The file names specified in @p vertexFile and @p fragmentFile are relative paths to the shaders
+     * resource file shipped together with KWin. This means this method can only be used for built-in
+     * effects, for 3rd party effects @link {generateCustomShader} should be used.
+     *
+     * If the @p vertexFile is empty a vertex shader with the given @p traits is generated.
+     * If it is not empty the @p vertexFile is used as the source for the vertex shader.
+     *
+     * The same applies for argument @p fragmentFile just for the fragment shader.
+     *
+     * So if both @p vertexFile and @p fragmentFile are provided the @p traits are ignored.
+     * If neither are provided a new shader following the @p traits is generated.
+     *
+     * @param traits The shader traits for generating the shader
+     * @param vertexFile optional vertex shader source code to be used instead of shader traits
+     * @param fragmentFile optional fragment shader source code to be used instead of shader traits
+     * @return new generated shader
+     * @see generateCustomShader
+     * @since 5.6
+     **/
+    GLShader *generateShaderFromResources(ShaderTraits traits, const QString &vertexFile = QString(), const QString &fragmentFile = QString());
+
     /**
      * Compiles and tests the dynamically generated shaders.
      * Returns true if successful and false otherwise.
      */
     bool selfTest();
 
-    /**
-     * Sets the virtual screen size to @p s.
-     * @since 5.2
-     **/
-    static void setVirtualScreenSize(const QSize &s) {
-        s_virtualScreenSize = s;
-    }
-
     /**
      * @return a pointer to the ShaderManager instance
      **/
@@ -388,8 +344,6 @@ private:
     ShaderManager();
     ~ShaderManager();
 
-    void initShaders();
-    void resetShader(ShaderType type);
     void bindFragDataLocations(GLShader *shader);
     void bindAttributeLocations(GLShader *shader) const;
 
@@ -398,14 +352,10 @@ private:
     GLShader *generateShader(ShaderTraits traits);
 
     QStack<GLShader*> m_boundShaders;
-    GLShader *m_shader[3];
     QHash<ShaderTraits, GLShader *> m_shaderHash;
-    bool m_inited;
-    bool m_valid;
     bool m_debug;
-    QByteArray m_shaderDir;
+    QString m_resourcePath;
     static ShaderManager *s_shaderManager;
-    static QSize s_virtualScreenSize;
 };
 
 /**
@@ -427,14 +377,6 @@ private:
 class KWINGLUTILS_EXPORT ShaderBinder
 {
 public:
-    /**
-     * @brief Pushes the Shader of the given @p type to the ShaderManager's stack.
-     *
-     * @param type The built-in Shader type
-     * @param reset Whether all uniforms should be reset to their default values. Defaults to false.
-     * @see ShaderManager::pushShader
-     **/
-    explicit ShaderBinder(ShaderManager::ShaderType type, bool reset = false);
     /**
      * @brief Pushes the given @p shader to the ShaderManager's stack.
      *
@@ -442,6 +384,14 @@ public:
      * @see ShaderManager::pushShader
      **/
     explicit ShaderBinder(GLShader *shader);
+    /**
+     * @brief Pushes the Shader with the given @p traits to the ShaderManager's stack.
+     *
+     * @param traits The traits describing the shader
+     * @see ShaderManager::pushShader
+     * @since 5.6
+     **/
+    explicit ShaderBinder(ShaderTraits traits);
     ~ShaderBinder();
 
     /**
@@ -453,13 +403,6 @@ private:
     GLShader *m_shader;
 };
 
-inline
-ShaderBinder::ShaderBinder(ShaderManager::ShaderType type, bool reset)
-    : m_shader(nullptr)
-{
-    m_shader = ShaderManager::instance()->pushShader(type, reset);
-}
-
 inline
 ShaderBinder::ShaderBinder(GLShader *shader)
     : m_shader(shader)
@@ -467,6 +410,13 @@ ShaderBinder::ShaderBinder(GLShader *shader)
     ShaderManager::instance()->pushShader(shader);
 }
 
+inline
+ShaderBinder::ShaderBinder(ShaderTraits traits)
+    : m_shader(nullptr)
+{
+    m_shader = ShaderManager::instance()->pushShader(traits);
+}
+
 inline
 ShaderBinder::~ShaderBinder()
 {
