@@ -20,6 +20,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef KWIN_MAIN_WAYLAND_H
 #define KWIN_MAIN_WAYLAND_H
 #include "main.h"
+#include <QProcessEnvironment>
+
+class QProcess;
 
 namespace KWin
 {
@@ -34,38 +37,42 @@ public:
     void setStartXwayland(bool start) {
         m_startXWayland = start;
     }
-    void setBackendSize(const QSize &size) {
-        m_backendSize = size;
+    void setApplicationsToStart(const QStringList &applications) {
+        m_applicationsToStart = applications;
     }
-    void setWindowed(bool set) {
-        m_windowed = set;
+    void setInputMethodServerToStart(const QString &inputMethodServer) {
+        m_inputMethodServerToStart = inputMethodServer;
     }
-    void setX11Display(const QByteArray &display) {
-        m_x11Display = display;
+    void setProcessStartupEnvironment(const QProcessEnvironment &environment) {
+        m_environment = environment;
     }
-    void setWaylandDisplay(const QByteArray &display) {
-        m_waylandDisplay = display;
+    void setSessionArgument(const QString &session) {
+        m_sessionArgument = session;
     }
-    void setFramebuffer(const QString &fbdev) {
-        m_framebuffer = fbdev;
+
+    QProcessEnvironment processStartupEnvironment() const override {
+        return m_environment;
     }
 
 protected:
     void performStartup() override;
+    void setupCrashHandler() override;
 
 private:
     void createBackend();
     void createX11Connection();
     void continueStartupWithScreens();
     void continueStartupWithX();
+    void startXwaylandServer();
 
     bool m_startXWayland = false;
     int m_xcbConnectionFd = -1;
-    QSize m_backendSize;
-    bool m_windowed = false;
-    QByteArray m_x11Display;
-    QByteArray m_waylandDisplay;
-    QString m_framebuffer;
+    QStringList m_applicationsToStart;
+    QString m_inputMethodServerToStart;
+    QProcess *m_xwaylandProcess = nullptr;
+    QMetaObject::Connection m_xwaylandFailConnection;
+    QProcessEnvironment m_environment;
+    QString m_sessionArgument;
 };
 
 }
