@@ -105,6 +105,7 @@ public:
      * @see findClient(Predicate, xcb_window_t)
      */
     Client *findClient(std::function<bool (const Client*)> func) const;
+    AbstractClient *findAbstractClient(std::function<bool (const AbstractClient*)> func) const;
     /**
      * @brief Finds the Client matching the given match @p predicate for the given window.
      *
@@ -115,6 +116,7 @@ public:
      */
     Client *findClient(Predicate predicate, xcb_window_t w) const;
     void forEachClient(std::function<void (Client*)> func);
+    void forEachAbstractClient(std::function<void (AbstractClient*)> func);
     Unmanaged *findUnmanaged(std::function<bool (const Unmanaged*)> func) const;
     /**
      * @brief Finds the Unmanaged with the given window id.
@@ -146,7 +148,7 @@ public:
      */
     AbstractClient* mostRecentlyActivatedClient() const;
 
-    Client* clientUnderMouse(int screen) const;
+    AbstractClient* clientUnderMouse(int screen) const;
 
     void activateClient(AbstractClient*, bool force = false);
     void requestFocus(AbstractClient* c, bool force = false);
@@ -323,7 +325,7 @@ public:
     int packPositionDown(const AbstractClient* cl, int oldy, bool bottom_edge) const;
 
     void cancelDelayFocus();
-    void requestDelayFocus(Client*);
+    void requestDelayFocus(AbstractClient*);
 
     /**
     * updates the mouse position to track whether a focus follow mouse focus change was caused by
@@ -533,7 +535,7 @@ private:
 
     // Delay(ed) window focus timer and client
     QTimer* delayFocusTimer;
-    Client* delayfocus_client;
+    AbstractClient* delayfocus_client;
     QPoint focusMousePos;
 
     ClientList clients;
